/*
 *
 *               PC-1360/K Binary file transfer support utility
 *
 *                      Copyright (C) 1993 by Y.Akagawa
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <dos.h>
#include "rs232c.h"

#define M_SEND	1
#define M_REC	2
#define M_TEXT  0
#define M_BIN   1

int send( FILE *fp );
int rec( FILE *fp );
int check_header( FILE *fp, unsigned int fs );
void check_cmdlin( int argc, char **argv );
int get( FILE *fp );
void prt_usage( void );

#ifdef PC98
const char *msg_title     = "File Transfer Utility BINRS Version 0.01/PC98";
const char *msg_fileerr   = "I[vł܂";
const char *msg_abort     = "f";
const char *msg_sending   = "|M|";
const char *msg_sendend   = "|M|";
const char *msg_request   = "\n%sbpsőMĂ";
const char *msg_receiving = "|M|";
const char *msg_recend    = "|M|";
const char *msg_optiona   = "i[JnAhXiPUij";
const char *msg_optionr   = "t@CM";
const char *msg_options   = "t@CM";
#else
const char *msg_title = "File Transfer Utility BINRS Version 0.01/DOS";
const char *msg_fileerr   = "Can't Open.";
const char *msg_abort     = " Abort.";
const char *msg_sending   = "- Sending -";
const char *msg_sendend   = " Complete! ";
const char *msg_request   = "\nPlease send by %sbps";
const char *msg_receiving = "- Receiving -";
const char *msg_recend    = " Complete!   ";
const char *msg_optiona   = "Load address (Hex)";
const char *msg_optionr   = "Send file";
const char *msg_options   = "Receive file";
#endif

int flg_mode,
	flg_txbn,
	flg_adr;

unsigned int start_addr,
			 run_addr,
			 bps;

char	*b[] = { "75", "150", "300", "600", "1200", "2400", "4800", 
				"9600", "19200", "38400" };

void main( int argc, char **argv )
{
	FILE *fp;
	int	res;
	char fmode[3];

	printf("%s\n", msg_title );
	printf("Copyright (c) 1993 by Y.Akagawa\n\n" );

	check_cmdlin( argc, argv );

	if( flg_mode == M_SEND )
	{
		strcpy( fmode, "rb" );
	}
	else
	{
		if( flg_mode == M_REC )
		{
			strcpy( fmode, "wb" );
		}
		else
		{
			prt_usage();
			exit( 1 );
		}
	}

	if( ( fp = fopen( argv[1], fmode ) ) == NULL )
	{
		printf("%s %s\n", argv[1], msg_fileerr );
		exit( 1 );
	}


	rs_init(bps, PARITY_NONE, STOP_1, CHR_8, XFLOW_OFF);
										/* MS-DOSėpłł́A̓_~[ */

	if( flg_mode == M_SEND )
		res = send( fp );
	else
		res = rec( fp );

	if( res == -1 )
		printf("\r%s\x1b[0K", msg_abort );
	fclose( fp );
}


/*********************/
/* File Send Routine */
/* Entry:			 */
/*  FILE *fp;		 */
/* Return:			 */
/* -1 : abort		 */
/*  0 : normal		 */
/*********************/

int send( FILE *fp )
{
	int dat, i;
	unsigned int fs;
	unsigned char header[17];

	printf("\n%s\x1b[0K", msg_sending);

	fs = (unsigned int)filelength( fileno( fp ) );	/* Get file size */

	if( flg_txbn == M_BIN )
	{
		i = check_header( fp, fs );
		fseek( fp, 0, SEEK_SET );
		if( i == -1 )							/* Header found ? */
		{
			header[0x00] = 0xff;				/* Header create */
			header[0x01] = 0x30;
			header[0x02] = 0x00;
			header[0x03] = 0x00;
			header[0x04] = 0x10;
			header[0x05] = (unsigned char)( fs & 0xff );
			header[0x06] = (unsigned char)( ( fs >> 8 ) & 0xff );
			header[0x07] = 0;
			header[0x08] = (unsigned char)( start_addr & 0xff );
			header[0x09] = (unsigned char)( ( start_addr >> 8 ) & 0xff );
			header[0x0a] = 0;
			header[0x0b] = 0x00;
			header[0x0c] = 0x00;
			header[0x0d] = 0x00;
			header[0x0e] = 0x00;
			header[0x0f] = 0x00;
		}
		else
		{
			for( i=0; i<16; i++ )
				header[i] = (unsigned char)fgetc( fp );
			fs = (unsigned int)header[0x05];
			fs += (unsigned int)header[0x06] * 0x100;
			if( flg_adr )
			{
				header[0x08] = (unsigned char)( start_addr & 0xff );
				header[0x09] = (unsigned char)( ( start_addr >> 8 ) & 0xff );
			}
		}

		printf("\x1b[1A\rload addr:%02X%02X size:%04X\n",
		 header[9],header[8], fs);

		for( i=0; i<16; i++ )					/* Send header */
		{
			if( putaux( header[i] ) == -2 )
				return( -1 );
		}

		while( ( dat = fgetc( fp ) ) != EOF )
		{
			if( putaux( dat ) == -2 )				/* Break ? */
				return( -1 );
		}
	}
	else
	{
		printf("\x1b[1A\rText mode. size:%04X\n", fs );
		while( ( dat = fgetc( fp ) ) != EOF )
		{
			if( putaux( dat ) == -2 )				/* Break ? */
				return( -1 );
			if( dat == 0x1a )
				break;
		}
		if( dat != 0x1a )
			putaux( 0x1a );
	}

	printf("\a\r%s", msg_sendend );

	return( 0 );
}


/************************/
/* File Receive Routine */
/* Entry:			    */
/*  FILE *fp;			*/
/* Return:			    */
/* -1 : abort			*/
/*  0 : normal			*/
/************************/

int rec( FILE *fp )
{

	unsigned int size, laddr, raddr;
	int dat;

	printf(msg_request, b[bps]);

	if( flg_txbn == M_BIN )
	{
		if( get( fp ) == -1 )
			return( -1 );
		if( get( fp ) == -1 )
			return( -1 );
		if( get( fp ) == -1 )
			return( -1 );
		if( get( fp ) == -1 )
			return( -1 );

		if( get( fp ) == -1 )
			return( -1 );

		size = (unsigned int)get( fp );
		size += (unsigned int)get( fp ) * 0x100;

		if( get( fp ) == -1 )
			return( -1 );

		laddr = (unsigned int)get( fp );
		laddr += (unsigned int)get( fp ) * 0x100;

		if( get( fp ) == -1 )
			return( -1 );

		raddr = (unsigned int)get( fp );
		raddr += (unsigned int)get( fp ) * 0x100;

		if( get( fp ) == -1 )
			return( -1 );
		if( get( fp ) == -1 )
			return( -1 );
		if( get( fp ) == -1 )
			return( -1 );

		printf("\x1b[1A\rload addr:%04X size:%04X\n", laddr, size);

		printf("\r%s\x1b[0K", msg_receiving );

		while( size-- )
		{
			dat = get( fp );
			if( dat == -1 )						/* break ? */
				return( -1 );
		}
	}
	else
	{
		printf("\x1b[1A\rText mode.\n");
		get( fp );
		printf("\r%s\x1b[0K", msg_receiving );

		do
		{
			dat = get( fp );
			if( dat == -1 )						/* break ? */
				return( -1 );
		}while( dat != 0x1a );
	}
	printf("\a\r%s\x1b[0K", msg_recend );

	return( 0 );
}


/******************************/
/* Command line check Routine */
/* Entry:					  */
/*       int argc;			  */
/*       char **argv;		  */
/******************************/

void check_cmdlin( int argc, char **argv )
{
	int  i;
	unsigned int b;
	char *stpstr;

	if( argc < 2 )
	{
		prt_usage();
		exit( -1 );
	}

	bps         =  BPS_4800;
	start_addr  =  0;
	run_addr    =  0;
	flg_mode    =  0;
	flg_txbn    =  M_BIN;
	flg_adr     =  0;

	if( argc > 2 )
	{
		for( i = 2; i < argc; i++ )
		{
			if( toupper( argv[i][1] ) == 'S' )
				flg_mode = M_SEND;

			if( toupper( argv[i][1] ) == 'R' )
				flg_mode = M_REC;

			if( toupper( argv[i][1] ) == 'A' )
			{
				flg_adr = 1;
				start_addr = (unsigned int)strtoul( argv[i]+2, &stpstr, 16 );
			}

#ifdef PC98
			if( toupper( argv[i][1] ) == 'T' )
			{
				flg_txbn = M_TEXT;
				bps = BPS_1200;
			}

			if( toupper( argv[i][1] ) == 'B' )
			{
				bps = (unsigned int) atoi( argv[i]+2 );
				b = bps / 100;
				if( bps != b * 100 )
					bps = 0;
				else
					bps = b;
				switch( bps )
				{
					case 3 :
						bps = BPS_300;
						break;
					case 6 :
						bps = BPS_600;
						break;
					case 12 :
						bps = BPS_1200;
						break;
					case 24 :
						bps = BPS_2400;
						break;
					case 48 :
						bps = BPS_4800;
						break;
					case 96 :
						bps = BPS_9600;
						break;
					case 192 :
						bps = BPS_19200;
						break;
					case 384 :
						bps = BPS_38400;
						break;
					default :
						prt_usage();
						exit( -1 );
				}
			}
#endif
		}
	}
}


/************************/
/* Header check Routine */
/* Entry:				*/
/*  FILE *fp;			*/
/*  long filesize;		*/
/* Return:				*/
/* -1 : no Header		*/
/*  0 : Header found	*/
/************************/

int check_header( FILE *fp, unsigned int fs )
{
	if( fs < 16 )
		return( -1 );
	if( fgetc( fp ) != 0xff )
		return( -1 );
	if( fgetc( fp ) != 0x30 )
		return( -1 );
	if( fgetc( fp ) != 0x00 )
		return( -1 );
	if( fgetc( fp ) != 0x00 )
		return( -1 );
	if( fgetc( fp ) != 0x10 )
		return( -1 );
	return( 0 );
}


/**************************/
/* Receive AUX & Put file */
/* Entry:				  */
/*  FILE *fp;			  */
/* Return:				  */
/* -1 : Break			  */
/*  * : Receive data	  */
/**************************/

int get( FILE *fp )
{
	int	dat;
	do
	{
		dat = getaux();
		if( dat == -2 )
			return( -1 );
	}while( dat == -1 );
	fputc( dat, fp );
	return( dat );
}

/**************************/
/* Display usage          */
/**************************/

void prt_usage( void )
{
	printf("Usage : BINRS filename [option]\n\n");
	printf("option : -Axxxx    %s\n", msg_optiona );
	printf("         -R        %s\n", msg_optionr );
	printf("         -S        %s\n", msg_options );
#ifdef PC98
	printf("         -Bxxxx    ]xi300`38400j\n");
	printf("         -T        eLXg[h\n");
#endif
}
